@@ -158,15 +158,6 @@ basep_widget_class_init (BasePWidgetClass *klass)
                                                              BASEP_SHOWN,
                                                              G_PARAM_READWRITE));
 
-        g_object_class_install_property (object_class,
-                                        PROP_LEVEL,
-                                        g_param_spec_enum ("level",
-                                                             _("level"),
-                                                             _("level"),
-							     PANEL_TYPE_BASE_PLEVEL,
-                                                             BASEP_LEVEL_DEFAULT,
-                                                             G_PARAM_READWRITE));
-
         g_object_class_install_property (object_class,
         			  	PROP_HIDEBUTTONS_ENABLED,
                                         g_param_spec_boolean ("hidebuttons_enabled",
@@ -254,9 +245,6 @@ basep_widget_set_property (GObject *object, guint prop_id, const GValue *value,
         	case PROP_STATE:
 			basep->state = g_value_get_enum (value);
 			break;
-		case PROP_LEVEL:
-			basep->level = g_value_get_enum (value);
-			break;
         	case PROP_HIDEBUTTONS_ENABLED:
 			basep->hidebuttons_enabled = g_value_get_boolean (value);
 			break;
@@ -292,6 +280,16 @@ basep_widget_realize (GtkWidget *w)
 			  basep->hidebutton_w,
 			  basep->hidebutton_s);
 
+	if (basep->strut_left != 0 ||
+	    basep->strut_right != 0 ||
+	    basep->strut_top != 0 ||
+	    basep->strut_bottom != 0) {
+		xstuff_set_wmspec_strut (w->window,
+					 basep->strut_left,
+					 basep->strut_right,
+					 basep->strut_top,
+					 basep->strut_bottom);
+	}
 	
 	klass = basep_widget_get_pos_class (basep);
 	g_return_if_fail (klass);
@@ -407,7 +405,7 @@ basep_widget_size_allocate (GtkWidget *widget,
 
 		w = allocation->width;
 		h = allocation->height;
-		klass->get_hide_size (basep, hide_orient, &w,&h);
+		klass->get_hide_size (basep, hide_orient, &w, &h);
 		allocation->width = w;
 		allocation->height = h;
 
@@ -462,6 +460,66 @@ basep_widget_size_allocate (GtkWidget *widget,
 	}
 
 	gtk_widget_size_allocate (basep->ebox, &challoc);
+
+	/* FIXME: this should be handled in a nicer way */
+	if (BORDER_IS_WIDGET (basep)) {
+		int left = 0, right = 0, top = 0, bottom = 0;
+		switch (BORDER_POS (basep->pos)->edge) {
+		case BORDER_LEFT:
+			if (basep->mode == BASEP_AUTO_HIDE)
+				left = global_config.minimized_size ;
+			else
+				left = basep->shown_alloc.width;
+			break;
+		case BORDER_RIGHT:
+			if (basep->mode == BASEP_AUTO_HIDE)
+				right = global_config.minimized_size ;
+			else
+				right = basep->shown_alloc.width;
+			break;
+		case BORDER_TOP:
+			if (basep->mode == BASEP_AUTO_HIDE)
+				top = global_config.minimized_size ;
+			else
+				top = basep->shown_alloc.height;
+			break;
+		case BORDER_BOTTOM:
+			if (basep->mode == BASEP_AUTO_HIDE)
+				bottom = global_config.minimized_size ;
+			else
+				bottom = basep->shown_alloc.height;
+			break;
+		}
+		if (basep->strut_left != left ||
+		    basep->strut_right != right ||
+		    basep->strut_top != top ||
+		    basep->strut_bottom != bottom) {
+			basep->strut_left = left;
+			basep->strut_right = right;
+			basep->strut_top = top;
+			basep->strut_bottom = bottom;
+			if (GTK_WIDGET_REALIZED (widget)) {
+				xstuff_set_wmspec_strut (widget->window,
+							 left, right,
+							 top, bottom);
+			}
+		}
+	} else if (basep->strut_left != 0 ||
+		   basep->strut_right != 0 ||
+		   basep->strut_top != 0 ||
+		   basep->strut_bottom != 0) {
+		/* If not border widget and it seems to be
+		 * set to some bad values just whack the
+		 * _NET_WM_STRUT */
+		basep->strut_left =
+			basep->strut_right =
+			basep->strut_top =
+			basep->strut_bottom = 0;
+		if (GTK_WIDGET_REALIZED (widget)) {
+			xstuff_delete_property (widget->window,
+						"_NET_WM_STRUT");
+		}
+	}
 }
 
 static void
@@ -538,19 +596,19 @@ basep_pos_get_hide_size (BasePWidget *basep,
 {
 
 #ifdef BASEP_WIDGET_DEBUG
-	printf ("basep_pos_get_hide_size %d\n", panel_gconf_global_config_get_int ("panel-minimized-size"));
+	printf ("basep_pos_get_hide_size %d\n", global_config.minimized_size);
 #endif
 	switch (hide_orient) {
 	case PANEL_ORIENT_UP:
 	case PANEL_ORIENT_DOWN:
 		*h = (basep->state == BASEP_AUTO_HIDDEN)
-			? panel_gconf_global_config_get_int ("panel-minimized-size")
+			? global_config.minimized_size
 			: get_requisition_height (basep->hidebutton_n);
 		break;
 	case PANEL_ORIENT_RIGHT:
 	case PANEL_ORIENT_LEFT:
 		*w = (basep->state == BASEP_AUTO_HIDDEN)
-			? panel_gconf_global_config_get_int ("panel-minimized-size")
+			? global_config.minimized_size
 			: get_requisition_width (basep->hidebutton_e);
 		break;
 	}
@@ -570,12 +628,12 @@ basep_pos_get_hide_pos (BasePWidget *basep,
 		break;
 	case PANEL_ORIENT_RIGHT:
 		*x += w - ((basep->state == BASEP_AUTO_HIDDEN)
-			   ? panel_gconf_global_config_get_int ("panel-minimized-size")
+			   ? global_config.minimized_size 
 			   : get_requisition_width (basep->hidebutton_w));
 		break;
 	case PANEL_ORIENT_DOWN:
 		*y += h - ((basep->state == BASEP_AUTO_HIDDEN)
-			   ? panel_gconf_global_config_get_int ("panel-minimized-size")
+			   ? global_config.minimized_size
 			   : get_requisition_height (basep->hidebutton_s));
 		break;
 	}
@@ -789,13 +847,16 @@ basep_widget_do_hiding(BasePWidget *basep, PanelOrient hide_orient,
 		break;
 	}
 
-	if(!panel_gconf_global_config_get_bool ("disable-animations") && step != 0) {
+	if(!global_config.disable_animations && step != 0) {
 		GTimeVal tval;
 		long start_secs;
 		long start_time;
 		long end_time;
 		long cur_time;
 
+#ifdef BASEP_WIDGET_DEBUG
+	printf ("We are sliding the panel since animations are turned ON\n");
+#endif
 		g_get_current_time(&tval);
 		
 		start_secs = tval.tv_sec;
@@ -893,7 +954,7 @@ basep_widget_do_showing(BasePWidget *basep, PanelOrient hide_orient,
 		break;
 	}
 	
-	if(!panel_gconf_global_config_get_bool ("disable-animations") && step != 0) {
+	if(!global_config.disable_animations && step != 0) {
 		int i;
 		GTimeVal tval;
 		long start_secs;
@@ -1107,6 +1168,17 @@ basep_widget_redo_window(BasePWidget *basep)
 
 	basep_widget_update_winhints (basep);
 
+	if (basep->strut_left != 0 ||
+	    basep->strut_right != 0 ||
+	    basep->strut_top != 0 ||
+	    basep->strut_bottom != 0) {
+		xstuff_set_wmspec_strut (widget->window,
+					 basep->strut_left,
+					 basep->strut_right,
+					 basep->strut_top,
+					 basep->strut_bottom);
+	}
+
 	gtk_drag_dest_set (widget, 0, NULL, 0, 0);
 
 	gtk_widget_map(widget);
@@ -1128,10 +1200,6 @@ basep_widget_instance_init (BasePWidget *basep)
 	GTK_WINDOW(basep)->allow_shrink = TRUE;
 	GTK_WINDOW(basep)->allow_grow = TRUE;
 
-#ifdef FIXME
-	GTK_WINDOW(basep)->auto_shrink = TRUE;
-#endif
-
 	/*don't let us close the window*/                                       
 	
 	g_signal_connect(G_OBJECT(basep),"delete_event",                    
@@ -1171,12 +1239,10 @@ basep_widget_instance_init (BasePWidget *basep)
 	g_object_set (G_OBJECT (basep),
 		      "mode", BASEP_EXPLICIT_HIDE,
 		      "state", BASEP_SHOWN,
-		      "level", BASEP_LEVEL_DEFAULT, 
 		      "hidebuttons_enabled", TRUE,
 		      "hidebutton_pixmaps_enabled", TRUE,
 		      NULL);
 
-	basep->avoid_on_maximize = TRUE;
 	basep->leave_notify_timer_tag = 0;
 	basep->autohide_inhibit = FALSE;
 	basep->drawers_open = 0;
@@ -1210,83 +1276,18 @@ basep_widget_show_hidebutton_pixmaps(BasePWidget *basep)
 void
 basep_widget_update_winhints (BasePWidget *basep)
 {
-  GtkWidget *w = GTK_WIDGET (basep);
-  gtk_window_set_decorated (GTK_WINDOW (w), FALSE);
-  gtk_window_stick (GTK_WINDOW (w));
-  /* FIXME: Should be GDK_WINDOW_TYPE_HINT_DOCK */
-  /*
-  gtk_window_set_type_hint (GTK_WINDOW (w),
-			    GDK_WINDOW_TYPE_HINT_TOOLBAR);
-  */
-#if FIXME /* Nearly all of this is old cruft and should just be removed. We need to use the new wm spec instead */
-	GnomeWinLayer layer;
-	guint coverhint;
+	GtkWidget *w = GTK_WIDGET (basep);
 
-        /* Do this before compliance check, the compliance
-         * check is for the legacy GNOME spec, this is for the
-         * new WM spec. Also, there's no harm to setting the hint
-         * for incompliant WM's, they'll just ignore it.
-         */
-        xstuff_set_wmspec_dock_hints (w->window);
-        
-	if ( ! basep->compliant_wm)
-		return;
+	gtk_window_set_decorated (GTK_WINDOW (w), FALSE);
+	gtk_window_stick (GTK_WINDOW (w));
 
-	gnome_win_hints_set_expanded_size (w, 0, 0, 0, 0);
-	gdk_window_set_decorations (w->window, 0);
-	gnome_win_hints_set_state (w, WIN_STATE_STICKY |
-				   WIN_STATE_FIXED_POSITION);
+	xstuff_set_wmspec_dock_hints (w->window);
 
-	if (basep->avoid_on_maximize) {
-		coverhint = WIN_HINTS_DO_NOT_COVER;
-	} else {
-		coverhint = 0;
-	}
+	/* FIXME: non-compliance should be tested! */
 
-	if (basep->level == BASEP_LEVEL_DEFAULT) {
-		if (global_config.normal_layer) {
-			layer = WIN_LAYER_NORMAL;
-		} else if (global_config.keep_bottom) {
-			layer = WIN_LAYER_BELOW;
-		} else {
-			layer = WIN_LAYER_DOCK;
-		}
-
-		/* drawers are always in DOCK, or NORMAL */
-		if ( DRAWER_IS_WIDGET(w) &&
-		     ! global_config.normal_layer)
-			layer = WIN_LAYER_DOCK;
-
-		/* if in autohiding mode, then we always want dock by default
-		 */
-		if (basep->mode == BASEP_AUTO_HIDE)
-			layer = WIN_LAYER_DOCK;
-
-	} else if (basep->level == BASEP_LEVEL_BELOW) {
-		layer = WIN_LAYER_BELOW;
-	} else if (basep->level == BASEP_LEVEL_NORMAL) {
-		layer = WIN_LAYER_NORMAL;
-	} else /* if (basep->level == BASEP_LEVEL_ABOVE) */ {
-		layer = WIN_LAYER_DOCK;
-	} 
-
-	switch (basep->state) {
-	case BASEP_SHOWN:
-	case BASEP_MOVING:
-
-		gnome_win_hints_set_layer (w, layer);
-		gnome_win_hints_set_hints (w, GNOME_PANEL_HINTS |
-					   coverhint);
-		break;
-	default: /* all of the hidden states */
-		gnome_win_hints_set_hints (w, GNOME_PANEL_HINTS);
-		gnome_win_hints_set_layer (w, (global_config.keep_bottom ||
-					       global_config.normal_layer)
-					   ? WIN_LAYER_ONTOP 
-					   : WIN_LAYER_ABOVE_DOCK);
-		break;
-	}
-#endif
+	if (BASEP_POS_GET_CLASS (basep)->update_winhints != NULL)
+		BASEP_POS_GET_CLASS (basep)->update_winhints (basep);
+	
 }
 
 void
@@ -1410,12 +1411,10 @@ basep_widget_construct (BasePWidget *basep,
 			gboolean packed,
 			gboolean reverse_arrows,
 			int screen,
-			PanelOrientation orient,
+			GtkOrientation orient,
 			int sz,
 			BasePMode mode,
 			BasePState state,
-			BasePLevel level,
-			gboolean avoid_on_maximize,
 			gboolean hidebuttons_enabled,
 			gboolean hidebutton_pixmaps_enabled,
 			PanelBackType back_type,
@@ -1502,9 +1501,6 @@ basep_widget_construct (BasePWidget *basep,
 
 	basep->screen = screen;
 
-	basep->level = level;
-	basep->avoid_on_maximize = avoid_on_maximize;
-
 	basep->hidebuttons_enabled = hidebuttons_enabled;
 	basep->hidebutton_pixmaps_enabled = hidebutton_pixmaps_enabled;
 
@@ -1532,12 +1528,10 @@ basep_widget_construct (BasePWidget *basep,
 void
 basep_widget_change_params (BasePWidget *basep,
 			    int screen,
-			    PanelOrientation orient,
+			    GtkOrientation orient,
 			    int sz,
 			    BasePMode mode,
 			    BasePState state,
-			    BasePLevel level,
-			    gboolean avoid_on_maximize,
 			    gboolean hidebuttons_enabled,
 			    gboolean hidebutton_pixmaps_enabled,
 			    PanelBackType back_type,
@@ -1589,13 +1583,6 @@ basep_widget_change_params (BasePWidget *basep,
 				   rotate_pixmap_bg,
 				   back_color);
 
-	if (basep->level != level ||
-	    basep->avoid_on_maximize != avoid_on_maximize) {
-		basep->level = level;
-		basep->avoid_on_maximize = avoid_on_maximize;
-		basep_widget_update_winhints (basep);
-	}
-
 	basep_widget_set_hidebuttons (basep);
 	basep_widget_show_hidebutton_pixmaps (basep);
 
@@ -1730,7 +1717,7 @@ basep_widget_explicit_hide (BasePWidget *basep, BasePState state)
 		basep_widget_update_winhints (basep);
 		basep_widget_do_hiding (basep, hide_orient,
 					size, 
-					panel_gconf_global_config_get_int ("panel-hide-speed"));
+					global_config.animation_speed);
 	}
 
 	basep->state = state;
@@ -1770,7 +1757,8 @@ basep_widget_explicit_show (BasePWidget *basep)
 		basep_widget_update_winhints (basep);
 		basep_widget_do_showing (basep, hide_orient,
 					 size, 
-					 panel_gconf_global_config_get_int ("panel-hide-speed"));
+					 global_config.animation_speed);
+					 
 	}
 	
 	basep->state = BASEP_SHOWN;
@@ -1820,7 +1808,7 @@ basep_widget_autoshow (gpointer data)
 		basep_widget_do_showing (basep,
 					 hide_orient,
 					 size,
-					 panel_gconf_global_config_get_int ("panel-hide-speed"));
+					 global_config.animation_speed);
 	}
 
 	basep->state = BASEP_SHOWN;
@@ -1870,12 +1858,12 @@ basep_widget_queue_autoshow (BasePWidget *basep)
 	}
 
 
-	if (panel_gconf_global_config_get_int ("panel-hide-delay") == 0) {
+	if (global_config.hide_delay == 0) {
 		basep_widget_autoshow (basep);
 	} else {
 		/* set up our delay for popup. */
 		basep->enter_notify_timer_tag =
-			gtk_timeout_add (panel_gconf_global_config_get_int ("panel-show-delay"),
+			gtk_timeout_add (global_config.show_delay,
 					 basep_widget_autoshow, basep);
 	} 
 }
@@ -1944,7 +1932,7 @@ basep_widget_autohide (gpointer data)
 		basep_widget_do_hiding (basep,
 					hide_orient,
 					size,
-					panel_gconf_global_config_get_int ("panel-hide-speed"));
+					global_config.animation_speed);
 	}
 
 
@@ -1989,7 +1977,7 @@ basep_widget_queue_autohide(BasePWidget *basep)
                 
        /* set up our delay for popup. */
         basep->leave_notify_timer_tag =
-                gtk_timeout_add (panel_gconf_global_config_get_int ("panel-hide-delay"),
+                gtk_timeout_add (global_config.hide_delay,
                                  basep_widget_autohide, basep);
 }
 
@@ -2286,7 +2274,7 @@ basep_calculate_borders (int screen)
 		if (EDGE_IS_WIDGET (basep)) {
 			BasePState state = basep->state;
 			if (PANEL_WIDGET (basep->panel)->orient ==
-			    PANEL_VERTICAL) {
+			    GTK_ORIENTATION_VERTICAL) {
 				if (sb->borders[edge].left < chreq.width &&
 				    state != BASEP_HIDDEN_RIGHT)
 					sb->borders[edge].left = chreq.width;
@@ -2312,7 +2300,7 @@ basep_calculate_borders (int screen)
 		} else /* ALIGNED */ {
 			AlignedAlignment align = ALIGNED_POS(basep->pos)->align;
 			if (PANEL_WIDGET (basep->panel)->orient ==
-			    PANEL_VERTICAL) {
+			    GTK_ORIENTATION_VERTICAL) {
 				if (align == ALIGNED_LEFT &&
 				    sb->borders[edge].left < chreq.width)
 					sb->borders[edge].left = chreq.width;
@@ -2431,16 +2419,12 @@ queue_recalc_handler (gpointer data)
 void
 basep_border_queue_recalc (int screen)
 {
-#ifdef BASEP_WIDGET_DEBUG
-	printf ("basep_border_queue_recalc\n");
-#endif
-	if (g_list_find (recalc_list,
-			 GINT_TO_POINTER (screen)) == NULL)
+	if (!g_list_find (recalc_list, GINT_TO_POINTER (screen)))
 		recalc_list = g_list_prepend (recalc_list,
 					      GINT_TO_POINTER (screen));
-	if (queue_recalc_id == 0) {
-		queue_recalc_id = gtk_idle_add (queue_recalc_handler, NULL);
-	}
+
+	if (!queue_recalc_id)
+		queue_recalc_id = g_idle_add (queue_recalc_handler, NULL);
 }
 
 void
