@@ -6,9 +6,10 @@
  */
 
 #include <config.h>
-#include <gnome.h>
 
-#include "panel-include.h"
+#include <libgnome/libgnome.h>
+
+#include "distribution.h"
 
 /* Note for distribution vendors:
  *
@@ -18,20 +19,12 @@
  *
  */
 
-static void rh_menu_init_func (void);
-static void rh_menu_show_func (GtkWidget *, GtkMenuItem *);
-
 static DistributionInfo distribution_info [] = {
 	{ DISTRIBUTION_DEBIAN, "/etc/debian_version",
 	  N_("Debian GNU/Linux"), N_("Debian menus"),
 	  "gnome-debian.png", "/var/lib/gnome/Debian/.",
 	  NULL, NULL
 	},
-	{ DISTRIBUTION_REDHAT, "/etc/redhat-release",
-	  N_("Red Hat Linux"), N_("Red Hat menus"), NULL,
-	  "apps-redhat",
-	  rh_menu_init_func, rh_menu_show_func
-	},
 	{ DISTRIBUTION_SUSE, "/etc/SuSE-release",
 	  N_("SuSE Linux"), N_("SuSE menus"), "gnome-suse.png",
 	  GNOME_DATADIR "/gnome/distribution-menus/SuSE/.",
@@ -46,7 +39,7 @@ internal_get_distribution_type (void)
 	DistributionInfo *ptr;
 
 	for (ptr = distribution_info; ptr->type != DISTRIBUTION_UNKNOWN; ptr++)
-		if (panel_file_exists (ptr->version_file))
+		if (g_file_test (ptr->version_file, G_FILE_TEST_EXISTS))
 			return ptr->type;
 
 	return DISTRIBUTION_UNKNOWN;
@@ -100,21 +93,3 @@ get_distribution_info (void)
 
 	return cache;
 }
-
-/*
- * Distribution specific menu functions.
- */
-
-static void
-rh_menu_init_func (void)
-{
-	/* Use the fork version to create the Red Hat menus. */
-	create_rh_menu (TRUE);
-}
-
-static void
-rh_menu_show_func (GtkWidget *menuw, GtkMenuItem *menuitem)
-{
-	rh_submenu_to_display (menuw, menuitem);
-}
-
