@@ -6,15 +6,21 @@
  *
  */
 
-/* since IS_BASEP_WIDGET() is used throughout, it makes life easier if we
+/* since BASEP_IS_WIDGET() is used throughout, it makes life easier if we
  * have a GtkWindow of our own.
  */
 
 #include <config.h>
 #include <unistd.h>
+#include <string.h>
+#include <time.h>
+
+#include <libgnome/libgnome.h>
+#include <gconf/gconf-client.h>
 
 #include "foobar-widget.h"
 
+#include "main.h"
 #include "menu.h"
 #include "menu-util.h"
 #include "session.h"
@@ -25,59 +31,58 @@
 #include "panel-util.h"
 #include "drawer-widget.h"
 #include "gnome-run.h"
-#include "scroll-menu.h"
 #include "gwmh.h"
 #include "tasklist_icon.h"
 #include "multiscreen-stuff.h"
 
-#define SMALL_ICON_SIZE 20
+#define ICON_SIZE 20
 
 extern GlobalConfig global_config;
 extern GSList *panel_list;
 
 extern GtkTooltips *panel_tooltips;
 
-static void foobar_widget_class_init	(FoobarWidgetClass	*klass);
-static void foobar_widget_init		(FoobarWidget		*foo);
-static void foobar_widget_realize	(GtkWidget		*w);
-static void foobar_widget_destroy	(GtkObject		*o);
-static void foobar_widget_size_allocate	(GtkWidget		*w,
-					 GtkAllocation		*alloc);
+static void foobar_widget_class_init	(FoobarWidgetClass *klass);
+static void foobar_widget_instance_init (FoobarWidget *foo);
+static void foobar_widget_realize	(GtkWidget *w);
+static void foobar_widget_destroy	(GtkObject *o);
+static void foobar_widget_size_allocate	(GtkWidget *w,
+					 GtkAllocation *alloc);
 static gboolean foobar_leave_notify	(GtkWidget *widget,
 					 GdkEventCrossing *event);
 static gboolean foobar_enter_notify	(GtkWidget *widget,
 					 GdkEventCrossing *event);
-static void append_task_menu (FoobarWidget *foo, GtkMenuBar *menu_bar);
+static void append_task_menu (FoobarWidget *foo, GtkMenuShell *menu_bar);
 static void setup_task_menu (FoobarWidget *foo);
 
 static GList *foobars = NULL;
 
-static GtkWindowClass *parent_class = NULL;
+static GtkWindowClass *foobar_widget_parent_class = NULL;
 
-
-
-GtkType
+GType
 foobar_widget_get_type (void)
 {
-	static GtkType foobar_widget_type = 0;
+	static GType object_type = 0;
 
-	if (!foobar_widget_type) {
-		GtkTypeInfo foobar_widget_info = {
-			"FoobarWidget",
-			sizeof (FoobarWidget),
-			sizeof (FoobarWidgetClass),
-			(GtkClassInitFunc) foobar_widget_class_init,
-			(GtkObjectInitFunc) foobar_widget_init,
-			(GtkArgSetFunc) NULL,
-			(GtkArgGetFunc) NULL
+	if (object_type == 0) {
+		static const GTypeInfo object_info = {
+                    sizeof (FoobarWidgetClass),
+                    (GBaseInitFunc)         NULL,
+                    (GBaseFinalizeFunc)     NULL,
+                    (GClassInitFunc)        foobar_widget_class_init,
+                    NULL,                   /* class_finalize */
+                    NULL,                   /* class_data */
+                    sizeof (FoobarWidget),
+                    0,                      /* n_preallocs */
+                    (GInstanceInitFunc)     foobar_widget_instance_init
 		};
 
-		foobar_widget_type = gtk_type_unique (gtk_window_get_type (),
-						      &foobar_widget_info);
+		object_type = g_type_register_static (GTK_TYPE_WINDOW, "FoobarWidget", &object_info, 0);
+        	foobar_widget_parent_class = g_type_class_ref (GTK_TYPE_WINDOW);
 
 	}
 
-	return foobar_widget_type;
+	return object_type;
 }
 
 static void
@@ -86,8 +91,6 @@ foobar_widget_class_init (FoobarWidgetClass *klass)
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 	GtkObjectClass *object_class = GTK_OBJECT_CLASS (klass);
 
-        parent_class = gtk_type_class (gtk_window_get_type ());
-
 	object_class->destroy = foobar_widget_destroy;
 
 	widget_class->realize = foobar_widget_realize;
@@ -102,17 +105,43 @@ pixmap_menu_item_new (const char *text, const char *try_file)
 	GtkWidget *item;
 	GtkWidget *label;
 
-	item = gtk_pixmap_menu_item_new ();
+	item = gtk_image_menu_item_new ();
 
-	if (try_file && gnome_preferences_get_menus_have_icons ()) {
-		GtkWidget *pixmap;
-		pixmap = gnome_stock_pixmap_widget_at_size (
-			NULL, try_file, SMALL_ICON_SIZE, SMALL_ICON_SIZE);
+	/* FIXME: listen to this gconf client key */
+	if (try_file != NULL && panel_menu_have_icons ()) {
+		GtkWidget *image;
+		GdkPixbuf *pixbuf;
+		char *file;
 
-		if(pixmap) {
-			gtk_widget_show (pixmap);
-			gtk_pixmap_menu_item_set_pixmap
-				(GTK_PIXMAP_MENU_ITEM (item), pixmap);
+		if (g_path_is_absolute (try_file))
+			file = g_strdup (try_file);
+		else
+			file = panel_pixmap_discovery (try_file,
+						       TRUE /* fallback */);
+
+		pixbuf = gdk_pixbuf_new_from_file (file, NULL);
+		if (pixbuf != NULL &&
+		    (gdk_pixbuf_get_width (pixbuf) != ICON_SIZE ||
+		     gdk_pixbuf_get_height (pixbuf) != ICON_SIZE)) {
+			GdkPixbuf *scaled;
+
+			scaled = gdk_pixbuf_scale_simple (pixbuf,
+							  ICON_SIZE,
+							  ICON_SIZE,
+							  GDK_INTERP_BILINEAR);
+
+			gdk_pixbuf_unref (pixbuf);
+
+			pixbuf = scaled;
+		}
+
+		if (pixbuf != NULL) {
+			image = gtk_image_new_from_pixbuf (pixbuf);
+
+			gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
+						       image);
+
+			gdk_pixbuf_unref (pixbuf);
 		}
 	}
 
@@ -126,24 +155,24 @@ pixmap_menu_item_new (const char *text, const char *try_file)
 }
 
 static void
-add_tearoff (GtkMenu *menu)
+add_tearoff (GtkMenuShell *menu)
 {
 	GtkWidget *item;
 
-	if (!gnome_preferences_get_menus_have_tearoff ())
+	if (!panel_menu_have_tearoff ())
 		return;
 	
 	item = gtk_tearoff_menu_item_new ();
 	gtk_widget_show (item);
-	gtk_menu_prepend (menu, item);
+	gtk_menu_shell_prepend (menu, item);
 }
 
 static gboolean
 foobar_leave_notify (GtkWidget *widget,
 		     GdkEventCrossing *event)
 {
-	if (GTK_WIDGET_CLASS (parent_class)->leave_notify_event)
-		GTK_WIDGET_CLASS (parent_class)->leave_notify_event (widget,
+	if (GTK_WIDGET_CLASS (foobar_widget_parent_class)->leave_notify_event)
+		GTK_WIDGET_CLASS (foobar_widget_parent_class)->leave_notify_event (widget,
 								     event);
 
 	return FALSE;
@@ -153,8 +182,8 @@ static gboolean
 foobar_enter_notify (GtkWidget *widget,
 		     GdkEventCrossing *event)
 {
-	if (GTK_WIDGET_CLASS (parent_class)->enter_notify_event)
-		GTK_WIDGET_CLASS (parent_class)->enter_notify_event (widget,
+	if (GTK_WIDGET_CLASS (foobar_widget_parent_class)->enter_notify_event)
+		GTK_WIDGET_CLASS (foobar_widget_parent_class)->enter_notify_event (widget,
 								     event);
 
 	if (global_config.autoraise)
@@ -176,9 +205,9 @@ url_menu_item (const char *label, const char *url, const char *pixmap)
 	GtkWidget *item;
 	item = pixmap_menu_item_new (label, pixmap);
 	if (!label) gtk_widget_set_sensitive (item, FALSE);
-	gtk_signal_connect (GTK_OBJECT (item), "activate",
-			    GTK_SIGNAL_FUNC (url_show), 
-			    (gpointer *)url);
+	g_signal_connect (G_OBJECT (item), "activate",
+			  G_CALLBACK (url_show), 
+			  url);
 	return item;
 }
 
@@ -190,26 +219,32 @@ about_cb (GtkWidget *w, gpointer data)
 }
 #endif
 
+#ifdef FIXME
 static void
 gmc_client (GtkWidget *w, gpointer data)
 {
 	char *v[3] = { "gmc-client" };
 	v[1] = data;
 	if(gnome_execute_async (g_get_home_dir (), 2, v) < 0)
-		panel_error_dialog(_("Cannot execute the gmc-client program,\n"
+		panel_error_dialog("cannot_execute_gmc_client",
+				   _("Cannot execute the gmc-client program,\n"
 				     "perhaps gmc is not installed"));
 }
+#endif /* FIXME */
 
+#if 0
 static void
 gnomecal_client (GtkWidget *w, gpointer data)
 {
 	char *v[4] = { "gnomecal", "--view" };
 	v[2] = data;
 	if(gnome_execute_async (g_get_home_dir (), 3, v) < 0)
-		panel_error_dialog(_("Cannot execute the gnome calendar,\n"
+		panel_error_dialog("cannot_execute_gnome_calendar",
+				   _("Cannot execute the gnome calendar,\n"
 				     "perhaps it's not installed.\n"
 				     "It is in the gnome-pim package."));
 }
+#endif
 
 #if 0
 static GtkWidget *
@@ -219,54 +254,85 @@ append_gnome_menu (FoobarWidget *foo, GtkWidget *menu_bar)
 	GtkWidget *menu;
 	int i;
 	char *url[][3] = {
-		{ N_("News (www)"),                N_("http://gnotices.gnome.org/gnome-news/"),          "gnome-news.png" },
-		{ N_("FAQ (www)"),                 N_("http://www.gnome.org/gnomefaq/html/"),            GNOME_STOCK_PIXMAP_HELP },
-		{ N_("Mailing Lists (www)"),       N_("http://mail.gnome.org/mailman/listinfo/"),        GNOME_STOCK_PIXMAP_MAIL },
-		{ NULL, "" },
-		{ N_("Software (www)"),            N_("http://www.gnome.org/applist/list-martin.phtml"), GNOME_STOCK_PIXMAP_SAVE },
-		{ N_("Development (www)"),         N_("http://developer.gnome.org/"),                    "gnome-devel.png" },
-		{ N_("Bug Tracking System (www)"), N_("http://bugzilla.gnome.org/"),                         "bug-buddy.png" },
-		{ NULL }
+		{ 
+		  N_("News (www)"),
+		  N_("http://gnotices.gnome.org/gnome-news/"),
+		  "gnome-news.png"
+		},
+		{ 
+		  N_("FAQ (www)"),
+		  N_("http://www.gnome.org/gnomefaq/html/"),
+		  GTK_STOCK_HELP 
+		},
+		{ 
+		  N_("Mailing Lists (www)"),
+		  N_("http://mail.gnome.org/mailman/listinfo/"),
+		  GNOME_STOCK_MAIL
+		},
+		{ 
+		  NULL, ""
+		},
+		{ 
+		  N_("Software (www)"),
+		  N_("http://www.gnome.org/applist/list-martin.phtml"),
+		  GTK_STOCK_SAVE
+		},
+		{
+		  N_("Development (www)"),
+		  N_("http://developer.gnome.org/"),
+		  "gnome-devel.png"
+		},
+		{
+		  N_("Bug Tracking System (www)"),
+		  N_("http://bugzilla.gnome.org/"),
+		  "bug-buddy.png"
+		},
+		{ 
+		  NULL
+		}
 	};
 	
 	
-	menu = hack_scroll_menu_new ();
+	menu = gtk_menu_new ();
 	
 	for (i=0; url[i][1]; i++)
-		gtk_menu_append (GTK_MENU (menu),
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu),
 				 url_menu_item (_(url[i][0]), url[i][1],
 						url[i][2]));
 		
 	add_menu_separator (menu);
 
-	item = pixmap_menu_item_new (_("About GNOME"), GNOME_STOCK_MENU_ABOUT);
-	gtk_menu_append (GTK_MENU (menu), item);
-	gtk_signal_connect (GTK_OBJECT (item), "activate",
-			    GTK_SIGNAL_FUNC (about_cb), NULL);
+	item = pixmap_menu_item_new (_("About GNOME"), GNOME_STOCK_ABOUT);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	g_signal_connect (G_OBJECT (item), "activate",
+			  G_CALLBACK (about_cb), NULL);
 
-	add_tearoff (GTK_MENU (menu));
+	add_tearoff (GTK_MENU_SHELL (menu));
 
 	item = pixmap_menu_item_new ("", "gnome-spider.png");
 
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu);
-	gtk_menu_bar_append (GTK_MENU_BAR (menu_bar), item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), item);
 	return item;
 }
 #endif
 
+#ifdef FIXME
 static GtkWidget *
 append_gmc_item (GtkWidget *menu, const char *label, char *flag)
 {
 	GtkWidget *item;
 
 	item = gtk_menu_item_new_with_label (label);
-	gtk_menu_append (GTK_MENU (menu), item);
-	gtk_signal_connect (GTK_OBJECT (item), "activate",
-			    GTK_SIGNAL_FUNC (gmc_client), flag);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	g_signal_connect (G_OBJECT (item), "activate",
+			  G_CALLBACK (gmc_client), flag);
 
 	return item;
 }
+#endif /* FIXME */
 
+#ifdef FIXME
 static gboolean
 display_gmc_menu (void)
 {
@@ -275,7 +341,7 @@ display_gmc_menu (void)
 
 	if ( ! checked_path) {
 		char *gmc_client;
-		gmc_client = panel_is_program_in_path ("gmc-client");
+		gmc_client = g_find_program_in_path  ("gmc-client");
 
 		if (gmc_client == NULL)
 			got_gmc = FALSE;
@@ -294,7 +360,9 @@ display_gmc_menu (void)
 
 	return TRUE;
 }
+#endif /* FIXME */
 
+#ifdef FIXME
 static void
 desktop_selected (GtkWidget *widget, gpointer data)
 {
@@ -311,12 +379,13 @@ desktop_selected (GtkWidget *widget, gpointer data)
 			gtk_widget_hide (item);
 	}
 }
+#endif /* FIXME */
 
+#ifdef FIXME
 static void
 append_desktop_menu (GtkWidget *menu_bar)
 {
 	GtkWidget *menu, *item;
-	char *char_tmp;
 	int i;
 	static char *arrange[] = { 
 		N_("By Name"), "--arrange-desktop-icons=name",
@@ -329,7 +398,7 @@ append_desktop_menu (GtkWidget *menu_bar)
 	};
 	GList *gmc_menu_items = NULL;
 
-	menu = hack_scroll_menu_new ();
+	menu = gtk_menu_new ();
 
 	for (i=0; arrange[i]; i+=2)
 		append_gmc_item (menu, _(arrange[i]), arrange[i+1]);
@@ -338,11 +407,11 @@ append_desktop_menu (GtkWidget *menu_bar)
 	gmc_menu_items = g_list_prepend (gmc_menu_items, item);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu);
 
-	add_tearoff (GTK_MENU (menu));
+	add_tearoff (GTK_MENU_SHELL (menu));
 
-	menu = hack_scroll_menu_new ();
+	menu = gtk_menu_new ();
 
-	gtk_menu_append (GTK_MENU (menu), item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
 	item = add_menu_separator (menu);
 	gmc_menu_items = g_list_prepend (gmc_menu_items, item);
@@ -357,94 +426,156 @@ append_desktop_menu (GtkWidget *menu_bar)
 	item = add_menu_separator (menu);
 	gmc_menu_items = g_list_prepend (gmc_menu_items, item);
 	
-	char_tmp = panel_is_program_in_path ("xscreensaver");
-	if (char_tmp) {	
+	if (panel_is_program_in_path  ("xscreensaver")) {
 		item = pixmap_menu_item_new (_("Lock Screen"), 
 					       "gnome-lockscreen.png");
-		gtk_menu_append (GTK_MENU (menu), item);
-		gtk_signal_connect (GTK_OBJECT (item), "activate",
-				    GTK_SIGNAL_FUNC (panel_lock), 0);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+		g_signal_connect (G_OBJECT (item), "activate",
+				  G_CALLBACK (panel_lock), 0);
 		setup_internal_applet_drag(item, "LOCK:NEW");
-
-		g_free (char_tmp);
 	}
 
 	item = pixmap_menu_item_new (_("Log Out"), "gnome-term-night.png");
-	gtk_menu_append (GTK_MENU (menu), item);
-	gtk_signal_connect (GTK_OBJECT (item), "activate",
-			    GTK_SIGNAL_FUNC(panel_quit), 0);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	g_signal_connect (G_OBJECT (item), "activate",
+			  G_CALLBACK (panel_quit), 0);
 	setup_internal_applet_drag (item, "LOGOUT:NEW");
 
-	add_tearoff (GTK_MENU (menu));
+	add_tearoff (GTK_MENU_SHELL (menu));
 
 	item = gtk_menu_item_new_with_label (_(" Desktop "));
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu);
-	gtk_menu_bar_append (GTK_MENU_BAR (menu_bar), item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), item);
 
 	gtk_signal_connect_full (GTK_OBJECT (menu), "show",
-				 GTK_SIGNAL_FUNC (desktop_selected),
+				 G_CALLBACK (desktop_selected),
 				 NULL,
 				 gmc_menu_items,
 				 (GtkDestroyNotify) g_list_free,
 				 FALSE,
 				 FALSE);
 }
+#endif /* FIXME */
+
+static void
+append_actions_menu (GtkWidget *menu_bar)
+{
+	GtkWidget *menu, *item;
+
+	menu = gtk_menu_new ();
+
+
+	add_tearoff (GTK_MENU_SHELL (menu));
+
+	menu = gtk_menu_new ();
+
+	item = pixmap_menu_item_new (_("Run..."), "gnome-run.png");
+	gtk_tooltips_set_tip (panel_tooltips, item,
+			      _("Run applications, if you know the "
+				"correct command to type in"),
+			      NULL);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	g_signal_connect (G_OBJECT (item), "activate",
+			  G_CALLBACK (show_run_dialog), 0);
+	setup_internal_applet_drag (item, "RUN:NEW");
+
+	/* FIXME: search */
+	/* FIXME: shutdown or reboot */
+
+	if (panel_is_program_in_path  ("xscreensaver")) {
+		item = pixmap_menu_item_new (_("Lock Display"), 
+					       "gnome-lockscreen.png");
+		gtk_tooltips_set_tip (panel_tooltips, item,
+				      _("Protect your computer from "
+					"unauthorized use"),
+				      NULL);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+		g_signal_connect (G_OBJECT (item), "activate",
+				  G_CALLBACK (panel_lock), 0);
+		setup_internal_applet_drag(item, "LOCK:NEW");
+	}
+
+	item = pixmap_menu_item_new (_("Log Out"), "gnome-term-night.png");
+	gtk_tooltips_set_tip (panel_tooltips, item,
+			      _("Quit from the GNOME desktop"),
+			      NULL);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	g_signal_connect (G_OBJECT (item), "activate",
+			  G_CALLBACK (panel_quit), 0);
+	setup_internal_applet_drag (item, "LOGOUT:NEW");
+
+	add_tearoff (GTK_MENU_SHELL (menu));
+
+	item = gtk_menu_item_new_with_label (_("Actions"));
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), item);
+
+	panel_strech_events_to_toplevel (item,
+					 TRUE /* top */,
+					 FALSE /* right */,
+					 FALSE /* bottom */,
+					 FALSE /* left */);
+}
 
 static GtkWidget *
 append_folder_menu (GtkWidget *menu_bar, const char *label,
-		    const char *pixmap, gboolean system, const char *path)
+		    const char *pixmap, const char *path,
+		    gboolean strech_left,
+		    gboolean strech_top,
+		    gboolean strech_right)
 {
 	GtkWidget *item, *menu;
-	char *real_path;
 
-	real_path = system 
-		? gnome_unconditional_datadir_file (path)
-		: gnome_util_home_file (path);
-
-	if (real_path == NULL) {
-		g_warning (_("can't find real path"));
-		return NULL;
-	}
-
-	menu = create_fake_menu_at (real_path,
+	menu = create_fake_menu_at (path,
 				    FALSE /* applets */,
 				    FALSE /* launcher_add */,
 				    FALSE /* favourites_add */,
 				    label /* dir_name */,
 				    NULL /* pixmap_name */,
 				    FALSE /* title */);
-	g_free (real_path);
+#ifdef FIXME /*EVIL!!*/
 	if (path != NULL && strcmp (path, "apps") == 0)
 		/* This will add the add submenu thingie */
 		start_favourites_menu (menu, TRUE /* fake_submenus */);
+#endif
 
 	if (menu == NULL) {
 		g_warning (_("menu wasn't created"));
 		return NULL;
 	}
 
-	if (pixmap)
+	if (pixmap != NULL)
 		item = pixmap_menu_item_new (label, pixmap);
 	else
 		item = gtk_menu_item_new_with_label (label);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu);
-	gtk_menu_bar_append (GTK_MENU_BAR (menu_bar), item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), item);
 
-	gtk_signal_connect (GTK_OBJECT (menu), "show",
-			    GTK_SIGNAL_FUNC (submenu_to_display),
-			    NULL);
+	g_signal_connect (G_OBJECT (menu), "show",
+			  G_CALLBACK (submenu_to_display),
+			  NULL);
+
+	if (strech_left || strech_top || strech_right)
+		panel_strech_events_to_toplevel (item,
+						 strech_top,
+						 strech_right,
+						 FALSE,
+						 strech_left);
+		
 
 	return menu;
 }
 
+#if 0
 static void
 append_gnomecal_item (GtkWidget *menu, const char *label, const char *flag)
 {
 	GtkWidget *item = gtk_menu_item_new_with_label (label);
-	gtk_menu_append (GTK_MENU (menu), item);
-	gtk_signal_connect (GTK_OBJECT (item), "activate",
-			    GTK_SIGNAL_FUNC (gnomecal_client), (gpointer)flag);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	g_signal_connect (G_OBJECT (item), "activate",
+			  G_CALLBACK (gnomecal_client), flag);
 }
+#endif
 
 static void
 update_clock (FoobarWidget *foo)
@@ -486,6 +617,7 @@ update_clock (FoobarWidget *foo)
 	gtk_label_set_text (GTK_LABEL (foo->clock_label), hour);
 }
 
+#if 0
 static int
 timeout_cb (gpointer data)
 {
@@ -500,7 +632,9 @@ timeout_cb (gpointer data)
 
 	return TRUE;
 }
+#endif
 
+#if 0
 static void
 set_fooclock_format (GtkWidget *w, char *format)
 {
@@ -511,14 +645,16 @@ set_fooclock_format (GtkWidget *w, char *format)
 						_(format));
 	}
 }
+#endif
 
+#if 0
 static void
 append_format_item (GtkWidget *menu, const char *format)
 {
 	char hour[256];
 	GtkWidget *item;
 	struct tm *das_tm;
-	time_t das_time = 43200;
+	time_t das_time = 0;
 
 	das_tm = localtime (&das_time);
 	if (strftime (hour, sizeof(hour), _(format), das_tm) == 0) {
@@ -530,22 +666,27 @@ append_format_item (GtkWidget *menu, const char *format)
 	hour[sizeof(hour)-1] = '\0'; /* just for sanity */
 
 	item = gtk_menu_item_new_with_label (hour);
-	gtk_menu_append (GTK_MENU (menu), item);
-	gtk_signal_connect (GTK_OBJECT (item), "activate",
-			    GTK_SIGNAL_FUNC (set_fooclock_format),
-			    (gpointer)format);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	g_signal_connect (G_OBJECT (item), "activate",
+			  G_CALLBACK (set_fooclock_format),
+			  format);
 }
+#endif
 
+#if 0
 static void
 set_time_cb (GtkWidget *menu_item, char *path)
 {
 	char *v[2] = { path };
 	
 	if (gnome_execute_async (g_get_home_dir (), 1, v) < 0)
-		panel_error_dialog (_("Could not call time-admin\n"
-						  "Perhaps time-admin is not installed"));
+		panel_error_dialog ("could_not_call_time_admin",
+				    _("Could not call time-admin\n"
+				      "Perhaps time-admin is not installed"));
 }
+#endif
 
+#if 0
 static GtkWidget *
 append_clock_menu (FoobarWidget *foo, GtkWidget *menu_bar)
 {
@@ -567,21 +708,21 @@ append_clock_menu (FoobarWidget *foo, GtkWidget *menu_bar)
 		NULL
 	};
 
-	menu = hack_scroll_menu_new ();
+	menu = gtk_menu_new ();
 	
 #if 0 /* put back when evolution can do this */
 	item = gtk_menu_item_new_with_label (_("Add appointement..."));
-	gtk_menu_append (GTK_MENU (menu), item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
 	add_menu_separator (menu);
 #endif
 
-	time_admin_path = gnome_is_program_in_path ("time-admin");
+	time_admin_path = g_find_program_in_path  ("time-admin");
 	if (time_admin_path) {
 		item = gtk_menu_item_new_with_label (_("Set Time"));
-		gtk_signal_connect (GTK_OBJECT (item), "activate",
-						GTK_SIGNAL_FUNC (set_time_cb), time_admin_path);
-		gtk_menu_append (GTK_MENU (menu), item);
+		g_signal_connect (G_OBJECT (item), "activate",
+				  G_CALLBACK (set_time_cb), time_admin_path);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 		add_menu_separator (menu);
 	}
 
@@ -590,17 +731,17 @@ append_clock_menu (FoobarWidget *foo, GtkWidget *menu_bar)
 
 	add_menu_separator (menu);
 
-	menu2 = hack_scroll_menu_new ();
+	menu2 = gtk_menu_new ();
 	for (i=0; formats[i]; i++)
 		append_format_item (menu2, formats[i]);
 
-	add_tearoff (GTK_MENU (menu2));
+	add_tearoff (GTK_MENU_SHELL (menu2));
 
 	item = gtk_menu_item_new_with_label (_("Format"));
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu2);
-	gtk_menu_append (GTK_MENU (menu), item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
-	add_tearoff (GTK_MENU (menu));
+	add_tearoff (GTK_MENU_SHELL (menu));
 
 	item = gtk_menu_item_new ();
 
@@ -611,10 +752,11 @@ append_clock_menu (FoobarWidget *foo, GtkWidget *menu_bar)
 	gtk_container_add (GTK_CONTAINER (item), foo->clock_label);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu);
 
-	gtk_menu_bar_append (GTK_MENU_BAR (menu_bar), item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), item);
 
 	return item;
 }
+#endif /* 0 */
 
 void
 foobar_widget_global_set_clock_format (const char *format)
@@ -640,32 +782,11 @@ void
 foobar_widget_update_winhints (FoobarWidget *foo)
 {
 	GtkWidget *w = GTK_WIDGET (foo);
-	GnomeWinLayer layer;
 
-	if ( ! foo->compliant_wm)
-		return;
+	gtk_window_set_decorated (GTK_WINDOW (w), FALSE);
+	gtk_window_stick (GTK_WINDOW (w));
 
-	xstuff_set_pos_size (w->window,
-			     multiscreen_x (foo->screen),
-			     multiscreen_y (foo->screen),
-			     w->allocation.width,
-			     w->allocation.height);
-
-	gnome_win_hints_set_expanded_size (w, 0, 0, 0, 0);
-	gdk_window_set_decorations (w->window, 0);
-	gnome_win_hints_set_state (w, WIN_STATE_STICKY |
-				   WIN_STATE_FIXED_POSITION);
-	
-	gnome_win_hints_set_hints (w, GNOME_PANEL_HINTS |
-				   WIN_HINTS_DO_NOT_COVER);	
-	if (global_config.normal_layer) {
-		layer = WIN_LAYER_NORMAL;
-	} else if (global_config.keep_bottom) {
-		layer = WIN_LAYER_BELOW;
-	} else {
-		layer = WIN_LAYER_DOCK;
-	}
-	gnome_win_hints_set_layer (w, layer);
+	xstuff_set_wmspec_dock_hints (w->window);
 }
 
 static void
@@ -674,15 +795,22 @@ foobar_widget_realize (GtkWidget *w)
 	gtk_window_set_wmclass (GTK_WINDOW (w),
 				"panel_window", "Panel");
 
-	if (GTK_WIDGET_CLASS (parent_class)->realize)
-		GTK_WIDGET_CLASS (parent_class)->realize (w);
+	if (GTK_WIDGET_CLASS (foobar_widget_parent_class)->realize)
+		GTK_WIDGET_CLASS (foobar_widget_parent_class)->realize (w);
 
 	foobar_widget_update_winhints (FOOBAR_WIDGET (w));
 	xstuff_set_no_group_and_no_input (w->window);
 
 	setup_task_menu (FOOBAR_WIDGET (w));
+
+	xstuff_set_wmspec_strut (w->window,
+				 0 /* left */,
+				 0 /* right */,
+				 w->allocation.height /* top */,
+				 0 /* bottom */);
 }
 
+#ifdef FIXME
 static void
 programs_menu_to_display(GtkWidget *menu)
 {
@@ -700,15 +828,19 @@ programs_menu_to_display(GtkWidget *menu)
 		create_root_menu (menu, TRUE, flags, TRUE, FALSE, FALSE);
 	}
 }
+#endif
 
+#ifdef FIXME
 static void
 set_the_task_submenu (FoobarWidget *foo, GtkWidget *item)
 {
-	foo->task_menu = hack_scroll_menu_new ();
+	foo->task_menu = gtk_menu_new ();
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), foo->task_menu);
 	/*g_message ("setting...");*/
 }
+#endif /* FIXME */
 
+#ifdef FIXME
 static void
 focus_task (GtkWidget *w, GwmhTask *task)
 {
@@ -719,14 +851,16 @@ focus_task (GtkWidget *w, GwmhTask *task)
 	gwmh_task_raise (task);
 	gwmh_task_focus (task);
 }
+#endif /* FIXME */
 
+#ifdef FIXME
 static void
 add_task (GwmhTask *task, FoobarWidget *foo)
 {
 	GtkWidget *item, *label;
 	char *title = NULL;
 	int slen;
-	GtkWidget *pixmap  = NULL;
+	GtkWidget *image = NULL;
 
 	static GwmhDesk *desk = NULL;
 
@@ -753,12 +887,12 @@ add_task (GwmhTask *task, FoobarWidget *foo)
 		g_free (tmp);
 	}
 	
-	item = gtk_pixmap_menu_item_new ();
-	pixmap = get_task_icon (task, GTK_WIDGET (foo));
-	if (pixmap != NULL) {
-		gtk_widget_show (pixmap);
-		gtk_pixmap_menu_item_set_pixmap (GTK_PIXMAP_MENU_ITEM (item),
-						 pixmap);
+	item = gtk_image_menu_item_new ();
+	image = get_task_icon (task, GTK_WIDGET (foo));
+	if (image != NULL) {
+		gtk_widget_show (GTK_WIDGET (image));
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
+					       GTK_WIDGET (image));
 	}
 
 	label = gtk_label_new (title);
@@ -767,9 +901,9 @@ add_task (GwmhTask *task, FoobarWidget *foo)
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_container_add (GTK_CONTAINER (item), label);
 	g_hash_table_insert (foo->tasks, task, item);
-	gtk_signal_connect (GTK_OBJECT (item), "activate", 
-			    GTK_SIGNAL_FUNC (focus_task),
-			    task);
+	g_signal_connect (G_OBJECT (item), "activate", 
+			  G_CALLBACK (focus_task),
+			  task);
 	gtk_widget_show_all (item);
 	
 	if (!desk)
@@ -778,11 +912,13 @@ add_task (GwmhTask *task, FoobarWidget *foo)
 	if (task->desktop == desk->current_desktop &&
 	    task->harea   == desk->current_harea   &&
 	    task->varea   == desk->current_varea)
-		gtk_menu_prepend (GTK_MENU (foo->task_menu), item);
+		gtk_menu_shell_prepend (GTK_MENU_SHELL (foo->task_menu), item);
 	else
-		gtk_menu_append (GTK_MENU (foo->task_menu), item);
+		gtk_menu_shell_append (GTK_MENU_SHELL (foo->task_menu), item);
 }
+#endif /* FIXME */
 
+#ifdef FIXME
 static void
 create_task_menu (GtkWidget *w, gpointer data)
 {
@@ -811,7 +947,9 @@ create_task_menu (GtkWidget *w, gpointer data)
 	GTK_MENU_SHELL (GTK_MENU_ITEM (w)->submenu)->active = 1;
 	our_gtk_menu_position (GTK_MENU (GTK_MENU_ITEM (w)->submenu));
 }
+#endif /* FIXME */
 
+#ifdef FIXME
 static void
 destroy_task_menu (GtkWidget *w, gpointer data)
 {
@@ -822,48 +960,43 @@ destroy_task_menu (GtkWidget *w, gpointer data)
 	foo->tasks = NULL;
 	set_the_task_submenu (foo, w);
 }
+#endif /* FIXME */
 
+#ifdef FIXME
 static GtkWidget *
-get_default_pixmap (void)
+get_default_image (void)
 {
-	GtkWidget *widget;
-	static GdkPixmap *pixmap = NULL;
-	static GdkBitmap *mask   = NULL;
+	GtkWidget *image;
+	static GdkPixbuf *pixbuf = NULL;
 	static gboolean looked   = FALSE;
 
-	if ( ! looked) {
+	if (! looked) {
 		GdkPixbuf *pb = NULL, *scaled = NULL;
-		pb = gdk_pixbuf_new_from_file (GNOME_ICONDIR"/gnome-tasklist.png");
+		pb = gdk_pixbuf_new_from_file (GNOME_ICONDIR"/gnome-tasklist.png", NULL);
 		
 		if (pb != NULL) {
 			scaled = gdk_pixbuf_scale_simple (pb, 20, 20, 
 							  GDK_INTERP_BILINEAR);
 			gdk_pixbuf_unref (pb);
-		}
 
-		if (scaled != NULL) {
-			gdk_pixbuf_render_pixmap_and_mask (scaled,
-							   &pixmap, &mask, 128);
-			gdk_pixbuf_unref (scaled);
-			
-			if (pixmap != NULL)
-				gdk_pixmap_ref (pixmap);
-
-			if (mask != NULL)
-				gdk_bitmap_ref (mask);
+			pixbuf = scaled;
 		}
 
 		looked = TRUE;
 	}
-	if (pixmap != NULL)
-		widget = gtk_pixmap_new (pixmap, mask);
-	else
-		widget = gtk_label_new ("*");
-	gtk_widget_show (widget);
 
-	return widget;
+	if (pixbuf != NULL) {
+	    image = gtk_image_new_from_pixbuf (pixbuf);
+	    gtk_widget_show (image);
+
+	    return image;
+	}
+
+	return NULL;
 }
+#endif /* FIXME */
 
+#ifdef FIXME
 static void
 set_das_pixmap (FoobarWidget *foo, GwmhTask *task)
 {
@@ -872,25 +1005,27 @@ set_das_pixmap (FoobarWidget *foo, GwmhTask *task)
 
 	foo->icon_task = NULL;
 
-	if (foo->task_pixmap != NULL)
-		gtk_widget_destroy (foo->task_pixmap);
-	foo->task_pixmap = NULL;
+	if (foo->task_image != NULL)
+		gtk_widget_destroy (GTK_WIDGET (foo->task_image));
+	foo->task_image = NULL;
 
 	if (task != NULL) {
-		foo->task_pixmap = get_task_icon (task, GTK_WIDGET (foo));
+		foo->task_image = get_task_icon (task, GTK_WIDGET (foo));
 		foo->icon_task = task;
 	}
 
-	if (foo->task_pixmap == NULL) {
-		foo->task_pixmap = get_default_pixmap ();
+	if (foo->task_image == NULL) {
+		foo->task_image = get_default_image ();
 	}
 
-	if (foo->task_pixmap != NULL) {
+	if (foo->task_image != NULL) {
 		gtk_container_add (GTK_CONTAINER (foo->task_bin),
-				   foo->task_pixmap);
+				   GTK_WIDGET (foo->task_image));
 	}
 }
+#endif /* FIXME */
 
+#ifdef FIXME
 static gboolean
 task_notify (gpointer data,
 	     GwmhTask *task,
@@ -945,9 +1080,10 @@ task_notify (gpointer data,
 	}
 	return TRUE;
 }
+#endif /* FIXME */
 
 static void
-append_task_menu (FoobarWidget *foo, GtkMenuBar *menu_bar)
+append_task_menu (FoobarWidget *foo, GtkMenuShell *menu_bar)
 {
 	foo->task_item = gtk_menu_item_new ();
 
@@ -956,19 +1092,20 @@ append_task_menu (FoobarWidget *foo, GtkMenuBar *menu_bar)
 	gtk_widget_show (foo->task_bin);
 	gtk_container_add (GTK_CONTAINER (foo->task_item), foo->task_bin);
 
-	gtk_menu_bar_append (menu_bar, foo->task_item);
+	gtk_menu_shell_append (menu_bar, foo->task_item);
 }
 
 static void
 setup_task_menu (FoobarWidget *foo)
 {
+#ifdef FIXME
 	GList *tasks;
 	g_assert (foo->task_item != NULL);
 
-	gtk_signal_connect (GTK_OBJECT (foo->task_item), "select",
-			    GTK_SIGNAL_FUNC (create_task_menu), foo);
-	gtk_signal_connect (GTK_OBJECT (foo->task_item), "deselect",
-			    GTK_SIGNAL_FUNC (destroy_task_menu), foo);
+	g_signal_connect (G_OBJECT (foo->task_item), "select",
+			  G_CALLBACK (create_task_menu), foo);
+	g_signal_connect (G_OBJECT (foo->task_item), "deselect",
+			  G_CALLBACK (destroy_task_menu), foo);
 
 	set_the_task_submenu (foo, foo->task_item);
 
@@ -987,26 +1124,28 @@ setup_task_menu (FoobarWidget *foo)
 		set_das_pixmap  (foo, NULL);
 
 	foo->notify = gwmh_task_notifier_add (task_notify, foo);
+#endif
 }
 
 static void
-foobar_widget_init (FoobarWidget *foo)
+foobar_widget_instance_init (FoobarWidget *foo)
 {
 	/*gchar *path;*/
 	GtkWindow *window = GTK_WINDOW (foo);
 	/*GtkWidget *bufmap;*/
 	GtkWidget *menu_bar, *bar;
-	GtkWidget *menu, *menuitem;
+	/*GtkWidget *menu, *menuitem;*/
 	/*GtkWidget *align;*/
-	gint flags;
+	/*gint flags;*/
 
 	foo->screen = 0;
 
-	foo->task_item = NULL;
-	foo->task_menu = NULL;
-	foo->task_pixmap = NULL;
-	foo->task_bin = NULL;
-	foo->icon_task = NULL;
+	foo->tasks      = NULL;
+	foo->task_item  = NULL;
+	foo->task_menu  = NULL;
+	foo->task_image = NULL;
+	foo->task_bin   = NULL;
+	foo->icon_task  = NULL;
 
 	foo->clock_format = g_strdup (_("%H:%M"));
 	foo->clock_timeout = 0;
@@ -1020,10 +1159,12 @@ foobar_widget_init (FoobarWidget *foo)
 
 	window->allow_shrink = TRUE;
 	window->allow_grow   = TRUE;
+#ifdef FIXME
 	window->auto_shrink  = TRUE;
+#endif
 
-	gtk_signal_connect (GTK_OBJECT (foo), "delete_event",
-			    GTK_SIGNAL_FUNC (gtk_true), NULL);
+	g_signal_connect (G_OBJECT (foo), "delete_event",
+			  G_CALLBACK (gtk_true), NULL);
 
 	gtk_widget_set_uposition (GTK_WIDGET (foo),
 				  multiscreen_x (foo->screen),
@@ -1036,7 +1177,8 @@ foobar_widget_init (FoobarWidget *foo)
 	gtk_container_add(GTK_CONTAINER(foo->ebox), foo->hbox);
 
 #if 0	
-	path = gnome_pixmap_file ("panel/corner1.png");
+	path = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, 
+					  "panel/corner1.png", TRUE, NULL);
 	bufmap = gnome_pixmap_new_from_file (path);
 	g_free (path);
 	align = gtk_alignment_new (0.0, 0.0, 1.0, 0.0);
@@ -1045,11 +1187,10 @@ foobar_widget_init (FoobarWidget *foo)
 #endif
 
 	menu_bar = gtk_menu_bar_new ();	
-	gtk_menu_bar_set_shadow_type (GTK_MENU_BAR (menu_bar),
-				      GTK_SHADOW_NONE);
 	
 	
-	menuitem = pixmap_menu_item_new (_("Programs"),
+#if 0
+	menuitem = pixmap_menu_item_new (_("Applications"),
 					 "gnome-logo-icon-transparent.png");
 	flags = (get_default_menu_flags() & 
 		 ~(MAIN_MENU_SYSTEM_SUB | MAIN_MENU_USER | MAIN_MENU_USER_SUB |
@@ -1057,27 +1198,43 @@ foobar_widget_init (FoobarWidget *foo)
 		   MAIN_MENU_DESKTOP_SUB)) | MAIN_MENU_SYSTEM;
 	menu = create_root_menu (NULL, TRUE, flags, TRUE, FALSE, FALSE);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), menu);
-	gtk_menu_bar_append (GTK_MENU_BAR (menu_bar), menuitem);
-	gtk_signal_connect (GTK_OBJECT (menu), "show",
-			    GTK_SIGNAL_FUNC (programs_menu_to_display),
-			    NULL);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), menuitem);
+	g_signal_connect (G_OBJECT (menu), "show",
+			  G_CALLBACK (programs_menu_to_display),
+			  NULL);
 	foo->programs = menu;
+#endif
 
+	/* FIXME: need to append some other menus such as KDE
+	 * or whatever is not merged!!!!! */
+	foo->programs =
+		append_folder_menu (menu_bar, _("Applications"),
+				    "gnome-logo-icon-transparent.png",
+				    "programs:/",
+				    TRUE /* strech_left */,
+				    TRUE /* strech_top */,
+				    FALSE /* strech_right */);
+
+#if 0
 	foo->favorites =
 		append_folder_menu(menu_bar, _("Favorites"), NULL,
-				   FALSE, "apps");
+				   "favorites:/");
 	foo->settings =
-		append_folder_menu(menu_bar, _("Settings"),  NULL, TRUE,
-			           "gnome/apps/Settings");
+		append_folder_menu(menu_bar, _("Settings"),  NULL,
+			           "programs:/Settings/");
+#endif
+#ifdef FIXME
 	append_desktop_menu (menu_bar);
+#endif
+	append_actions_menu (menu_bar);
 
 	gtk_box_pack_start (GTK_BOX (foo->hbox), menu_bar, FALSE, FALSE, 0);
 	
 	
 	/* panel widget */
-	foo->panel = panel_widget_new (FALSE, PANEL_HORIZONTAL,
-				       SIZE_TINY, PANEL_BACK_NONE,
-				       NULL, FALSE, FALSE, FALSE, TRUE, NULL);
+	foo->panel = panel_widget_new (FALSE, GTK_ORIENTATION_HORIZONTAL,
+				       PANEL_SIZE_X_SMALL, PANEL_BACK_NONE,
+				       NULL, FALSE, FALSE, FALSE, NULL);
 	PANEL_WIDGET (foo->panel)->panel_parent = GTK_WIDGET (foo);
 	PANEL_WIDGET (foo->panel)->drop_widget = GTK_WIDGET (foo);
 
@@ -1086,7 +1243,8 @@ foobar_widget_init (FoobarWidget *foo)
 	gtk_object_set_data (GTK_OBJECT (menu_bar), "menu_panel", foo->panel);
 
 #if 0
-	path = gnome_pixmap_file ("panel/corner2.png");
+	path = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, 
+					  "panel/corner2.png", TRUE, NULL);
 	bufmap = gnome_pixmap_new_from_file (path);
 	g_free (path);
 	align = gtk_alignment_new (1.0, 0.0, 1.0, 0.0);
@@ -1095,14 +1253,12 @@ foobar_widget_init (FoobarWidget *foo)
 #endif
 
 	bar = menu_bar = gtk_menu_bar_new ();
-	gtk_menu_bar_set_shadow_type (GTK_MENU_BAR (menu_bar),
-				      GTK_SHADOW_NONE);
-	append_clock_menu (foo, menu_bar);
 #if 0
+	append_clock_menu (foo, menu_bar);
 	/* TODO: use the gnome menu if no gnome compliant WM or tasklist disabled */
 	append_gnome_menu (foo, menu_bar);
 #endif
-	append_task_menu (foo, GTK_MENU_BAR (bar));
+	append_task_menu (foo, GTK_MENU_SHELL (bar));
 
 
 	gtk_box_pack_end (GTK_BOX (foo->hbox), menu_bar, FALSE, FALSE, 0);
@@ -1122,7 +1278,7 @@ queue_panel_resize (gpointer data, gpointer user_data)
 
 	g_return_if_fail (GTK_IS_WIDGET (panel));
 
-	if (!IS_DRAWER_WIDGET (panel) && !IS_FOOBAR_WIDGET (panel))
+	if (!DRAWER_IS_WIDGET (panel) && !FOOBAR_IS_WIDGET (panel))
 		gtk_widget_queue_resize (panel);
 }
 
@@ -1147,19 +1303,21 @@ foobar_widget_destroy (GtkObject *o)
 	if (foo->tasks != NULL)
 		g_hash_table_destroy (foo->tasks);
 	foo->tasks = NULL;
+#ifdef FIXME
 	if (foo->notify > 0)
 		gwmh_task_notifier_remove (foo->notify);
 	foo->notify = 0;
+#endif
 
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		GTK_OBJECT_CLASS (parent_class)->destroy (o);
+	if (GTK_OBJECT_CLASS (foobar_widget_parent_class)->destroy)
+		GTK_OBJECT_CLASS (foobar_widget_parent_class)->destroy (o);
 }
 
 static void
 foobar_widget_size_allocate (GtkWidget *w, GtkAllocation *alloc)
 {
-	if (GTK_WIDGET_CLASS (parent_class)->size_allocate)
-		GTK_WIDGET_CLASS (parent_class)->size_allocate (w, alloc);
+	if (GTK_WIDGET_CLASS (foobar_widget_parent_class)->size_allocate)
+		GTK_WIDGET_CLASS (foobar_widget_parent_class)->size_allocate (w, alloc);
 
 	if (GTK_WIDGET_REALIZED (w)) {
 		FoobarWidget *foo = FOOBAR_WIDGET (w);
@@ -1171,6 +1329,12 @@ foobar_widget_size_allocate (GtkWidget *w, GtkAllocation *alloc)
 
 		g_slist_foreach (panel_list, queue_panel_resize, NULL);
 		basep_border_queue_recalc (foo->screen);
+
+		xstuff_set_wmspec_strut (w->window,
+					 0 /* left */,
+					 0 /* right */,
+					 alloc->height /* top */,
+					 0 /* bottom */);
 	}
 }
 
@@ -1184,7 +1348,7 @@ foobar_widget_new (int screen)
 	if (foobar_widget_exists (screen))
 		return NULL;
 
-	foo = gtk_type_new (TYPE_FOOBAR_WIDGET);
+	foo = gtk_type_new (FOOBAR_TYPE_WIDGET);
 
 	foo->screen = screen;
 	gtk_widget_set_uposition (GTK_WIDGET (foo),
@@ -1253,7 +1417,7 @@ static void
 reparent_button_widgets(GtkWidget *w, gpointer data)
 {
 	GdkWindow *newwin = data;
-	if (IS_BUTTON_WIDGET (w)) {
+	if (BUTTON_IS_WIDGET (w)) {
 		ButtonWidget *button = BUTTON_WIDGET(w);
 		/* we can just reparent them all to 0,0 as the next thing
 		 * that will happen is a queue_resize and on size allocate
@@ -1343,6 +1507,12 @@ foobar_widget_redo_window(FoobarWidget *foo)
 
 	foobar_widget_update_winhints (foo);
 
+	xstuff_set_wmspec_strut (widget->window,
+				 0 /* left */,
+				 0 /* right */,
+				 widget->allocation.height /* top */,
+				 0 /* bottom */);
+
 	gtk_drag_dest_set (widget, 0, NULL, 0, 0);
 
 	gtk_widget_map(widget);
