@@ -1,9 +1,12 @@
 #ifndef PANEL_UTIL_H
 #define PANEL_UTIL_H
 
-#include <gnome.h>
+#include <gtk/gtk.h>
+
+#include "panel-widget.h"
+
+G_BEGIN_DECLS
 
-BEGIN_GNOME_DECLS
 
 typedef void (*UpdateFunction) (gpointer);
 
@@ -31,7 +34,8 @@ GtkWidget *	create_icon_entry	(GtkWidget *table,
 					 UpdateFunction func,
 					 gpointer data);
 
-void		panel_show_help		(const char *path);
+void		panel_show_help		(const char *path,
+					 const char *linkid);
 
 GList *		my_g_list_swap_next	(GList *list,
 					 GList *dl);
@@ -72,24 +76,62 @@ gboolean	convert_string_to_keysym_state (const char *string,
 char *		convert_keysym_state_to_string (guint keysym,
 						guint state);
 
-void		panel_set_dialog_layer	(GtkWidget *dialog);
-void		panel_reset_dialog_layers (void);
-GtkWidget *	panel_error_dialog	(const char *format, ...) G_GNUC_PRINTF (1, 2);
+GtkWidget *	panel_error_dialog	(const char *class,
+					 const char *format,
+					 ...) G_GNUC_PRINTF (2, 3);
+GtkWidget *	panel_info_dialog	(const char *class,
+					 const char *format,
+					 ...) G_GNUC_PRINTF (2, 3);
 
 gboolean	is_ext			(const char *file,
 					 const char *ext);
-int		strcasecmp_no_locale	(const char *s1,
-					 const char *s2);
-
-/* stolen from gnome-libs head as they are faster and don't use "stat" */
-gboolean	panel_file_exists	(const char *filename);
-char *		panel_is_program_in_path (const char *program);
+gboolean	is_ext2			(const char *file,
+					 const char *ext1,
+					 const char *ext2);
 
 int		find_applet		(GtkWidget *widget);
 
 int		get_requisition_width	(GtkWidget *widget);
 int		get_requisition_height	(GtkWidget *widget);
 
-END_GNOME_DECLS
+char *		panel_gnome_kde_help_path (const char *docpath);
+
+gboolean	panel_is_url		(const char *url);
+
+/* Quote string so that it could be on a shell like
+ * quoted line as an argument */
+char *		panel_quote_string	(const char *str);
+
+void		panel_push_window_busy	(GtkWidget *window);
+void		panel_pop_window_busy	(GtkWidget *window);
+
+gboolean	panel_is_program_in_path (const char *program);
+
+char *		panel_pixmap_discovery	(const char *name,
+					 gboolean fallback);
+
+void		panel_strech_events_to_toplevel (GtkWidget *widget,
+						 gboolean top,
+						 gboolean right,
+						 gboolean bottom,
+						 gboolean left);
+
+
+
+/* GnomeVFS reading utils */
+
+typedef struct _ReadBuf ReadBuf;
+
+int		readbuf_getc		(ReadBuf *rb);
+/* Note, does not include the trailing \n */
+char *		readbuf_gets		(char *buf, 
+					 gsize bufsize,
+					 ReadBuf *rb);
+ReadBuf *	readbuf_open		(const char *uri);
+/* unused for now */
+gboolean	readbuf_rewind		(ReadBuf *rb);
+void		readbuf_close		(ReadBuf *rb);
+
+G_END_DECLS
 
 #endif /* PANEL_UTIL_H */
