@@ -35,21 +35,17 @@ typedef struct _PanelWidgetClass	PanelWidgetClass;
 typedef struct _AppletRecord		AppletRecord;
 typedef struct _AppletData		AppletData;
 typedef struct _DNDRecord		DNDRecord;
-typedef enum {
-	PANEL_HORIZONTAL=0,
-	PANEL_VERTICAL
-} PanelOrientation;
-typedef enum {
-	PANEL_SWITCH_MOVE = 0,
-	PANEL_FREE_MOVE,
-	PANEL_PUSH_MOVE
-} PanelMovementType;
 
 struct _AppletData
 {
 	GtkWidget *	applet;
 	int		pos;
 	int		cells;
+	int             min_cells;
+
+	gboolean        expand_major;
+	gboolean        expand_minor;
+	
 	gboolean	dirty;
 
 	int		drag_off; /* offset on the applet where drag
@@ -70,7 +66,7 @@ struct _PanelWidget
 	GList			*no_window_applet_list;
 
 	int			size;
-	PanelOrientation	orient;
+	GtkOrientation		orient;
 	int			sz;
 	gboolean		fit_pixmap_bg;  /* fit pixmap while keeping
 						   ratio*/
@@ -114,7 +110,7 @@ struct _PanelWidgetClass
 	GtkFixedClass parent_class;
 
 	void (* orient_change) (PanelWidget *panel,
-				PanelOrientation orient);
+				GtkOrientation orient);
 	void (* size_change) (PanelWidget *panel,
 			      int sz);
 	void (* applet_move) (PanelWidget *panel,
@@ -136,7 +132,7 @@ struct _PanelWidgetClass
 GType		panel_widget_get_type		(void) G_GNUC_CONST;
 
 GtkWidget *	panel_widget_new		(gboolean packed,
-						 PanelOrientation orient,
+						 GtkOrientation orient,
 						 int sz,
 						 PanelBackType back_type,
 						 char *back_pixmap,
@@ -150,8 +146,10 @@ int		panel_widget_add_full		(PanelWidget *panel,
 						 GtkWidget *applet,
 						 int pos,
 						 gboolean bind_lower_events,
-						 gboolean insert_at_pos);
-#define panel_widget_add(panel,applet,pos) (panel_widget_add_full(panel,applet,pos,TRUE,FALSE))
+						 gboolean insert_at_pos,
+						 gboolean expand_major,
+						 gboolean expand_minor);
+#define panel_widget_add(panel,applet,pos) (panel_widget_add_full(panel,applet,pos,TRUE,FALSE,FALSE,FALSE))
 
 PanelWidget *	panel_widget_get_by_id		(gint32 id);
 void		panel_widget_set_id		(PanelWidget *panel,
@@ -197,7 +195,7 @@ void		panel_widget_applet_drag_end_no_grab(PanelWidget *panel);
 
 /* changing parameters */
 void		panel_widget_change_params	(PanelWidget *panel,
-						 PanelOrientation orient,
+						 GtkOrientation orient,
 						 int sz,
 						 PanelBackType back_type,
 						 char *pixmap_name,
@@ -208,7 +206,7 @@ void		panel_widget_change_params	(PanelWidget *panel,
 
 /* changing parameters (orient only) */
 void		panel_widget_change_orient	(PanelWidget *panel,
-						 PanelOrientation orient);
+						 GtkOrientation orient);
 
 void		panel_widget_set_back_pixmap	(PanelWidget *panel,
 						 char *file);
