@@ -22,11 +22,9 @@ struct _PerPanelConfig {
 	gboolean		hidebuttons;
 	gboolean		hidebutton_pixmaps;
 	BasePMode               mode;
-	BasePLevel		level;
-	gboolean		avoid_on_maximize;
 
 	/*floating widget*/
-	PanelOrientation        orient;
+	GtkOrientation        orient;
 	int                     x, y;
 
 	/*indexes for toggle*/
@@ -61,13 +59,7 @@ struct _PerPanelConfig {
 
 	GtkWidget		*pix_entry;
 
-	/*levels*/
-	GtkWidget               *level_menu;
-
-	/*avoid_on_maximize*/
-	GtkWidget               *avoid_on_maximize_button;
-
-	/*avoid_on_maximize*/
+	/*screen number spinner*/
 	GtkWidget               *screen_spin;
 
 	/*hiding stuff*/
@@ -104,8 +96,6 @@ void panel_config_register_changes (PerPanelConfig *ppc);
 
 void panel_config (GtkWidget *panel);
 void update_config_size (GtkWidget *panel);
-void update_config_level (BasePWidget *panel);
-void update_config_avoid_on_maximize (BasePWidget *panel);
 void update_config_screen (BasePWidget *panel);
 void update_config_mode (BasePWidget *panel);
 void update_config_hidebuttons (BasePWidget *panel);
@@ -121,7 +111,6 @@ void update_config_floating_pos_limits (BasePWidget *panel);
 void update_config_type (BasePWidget *panel);
 void kill_config_dialog (GtkWidget *panel);
 
-GtkWidget *make_level_widget (PerPanelConfig *ppc);
 GtkWidget *make_size_widget (PerPanelConfig *ppc);
 
 
