@@ -1,22 +1,21 @@
 #ifndef STATUS_H
 #define STATUS_H
 
-#include <gnome.h>
 #include "panel.h"
 #include "panel-types.h"
 #include "panel-widget.h"
-#include "gnome-panel.h"
+#include "GNOME_Panel.h"
 
-#include <libgnorba/gnorba.h>
+#include <bonobo-activation/bonobo-activation.h>
 
-BEGIN_GNOME_DECLS
+G_BEGIN_DECLS
 
 /*XXX we get some sore of conflict if we just use Status*/
 typedef struct _StatusApplet StatusApplet;
 struct _StatusApplet {
 	GtkWidget *frame;
 	GtkWidget *handle; /*a gtk handlebox but one that doesn't undock*/
-	PanelOrientation orient;
+	GtkOrientation orient;
 	int size;
 	AppletInfo *info;
 };
@@ -48,6 +47,6 @@ StatusSpot * status_applet_get_ss(guint32 winid);
 /*inhibit adding and updating for the purpose of quitting*/
 extern gboolean status_inhibit;
 
-END_GNOME_DECLS
+G_END_DECLS
 
 #endif
