@@ -24,7 +24,6 @@
 #include "swallow.h"
 
 #include "applet.h"
-#include "extern.h"
 #include "panel.h"
 #include "panel-util.h"
 #include "panel-widget.h"
@@ -328,7 +327,6 @@ ask_about_swallowing(PanelWidget *panel, int pos, gboolean exactpos)
 	gtk_widget_grab_focus(title_e);
 
 	gtk_widget_show_all (d);
-	panel_set_dialog_layer (d);
 }
 
 static int
